@@ -1,12 +1,14 @@
-__all__ = ['memmap']
+from __future__ import division, absolute_import, print_function
 
-import warnings
-from numeric import uint8, ndarray, dtype
-import sys
+import numpy as np
+from .numeric import uint8, ndarray, dtype
+from numpy.compat import long, basestring, is_pathlib_path
+
+__all__ = ['memmap']
 
 dtypedescr = dtype
 valid_filemodes = ["r", "c", "r+", "w+"]
-writeable_filemodes = ["r+","w+"]
+writeable_filemodes = ["r+", "w+"]
 
 mode_equivalents = {
     "readonly":"r",
@@ -16,17 +18,28 @@ mode_equivalents = {
     }
 
 class memmap(ndarray):
-    """
-    Create a memory-map to an array stored in a *binary* file on disk.
+    """Create a memory-map to an array stored in a *binary* file on disk.
 
     Memory-mapped files are used for accessing small segments of large files
-    on disk, without reading the entire file into memory.  Numpy's
+    on disk, without reading the entire file into memory.  NumPy's
     memmap's are array-like objects.  This differs from Python's ``mmap``
     module, which uses file-like objects.
 
+    This subclass of ndarray has some unpleasant interactions with
+    some operations, because it doesn't quite fit properly as a subclass.
+    An alternative to using this subclass is to create the ``mmap``
+    object yourself, then create an ndarray with ndarray.__new__ directly,
+    passing the object created in its 'buffer=' parameter.
+
+    This class may at some point be turned into a factory function
+    which returns a view into an mmap buffer.
+
+    Delete the memmap instance to close.
+
+
     Parameters
     ----------
-    filename : str or file-like object
+    filename : str, file-like object, or pathlib.Path instance
         The file name or file object to be used as the array data buffer.
     dtype : data-type, optional
         The data-type used to interpret the file contents.
@@ -49,35 +62,57 @@ class memmap(ndarray):
         Default is 'r+'.
     offset : int, optional
         In the file, array data starts at this offset. Since `offset` is
-        measured in bytes, it should be a multiple of the byte-size of
-        `dtype`. Requires ``shape=None``. The default is 0.
+        measured in bytes, it should normally be a multiple of the byte-size
+        of `dtype`. When ``mode != 'r'``, even positive offsets beyond end of
+        file are valid; The file will be extended to accommodate the
+        additional data. By default, ``memmap`` will start at the beginning of
+        the file, even if ``filename`` is a file pointer ``fp`` and
+        ``fp.tell() != 0``.
     shape : tuple, optional
-        The desired shape of the array. By default, the returned array will be
-        1-D with the number of elements determined by file size and data-type.
+        The desired shape of the array. If ``mode == 'r'`` and the number
+        of remaining bytes after `offset` is not a multiple of the byte-size
+        of `dtype`, you must specify `shape`. By default, the returned array
+        will be 1-D with the number of elements determined by file size
+        and data-type.
     order : {'C', 'F'}, optional
-        Specify the order of the ndarray memory layout: C (row-major) or
-        Fortran (column-major).  This only has an effect if the shape is
+        Specify the order of the ndarray memory layout:
+        :term:`row-major`, C-style or :term:`column-major`,
+        Fortran-style.  This only has an effect if the shape is
         greater than 1-D.  The default order is 'C'.
 
+    Attributes
+    ----------
+    filename : str or pathlib.Path instance
+        Path to the mapped file.
+    offset : int
+        Offset position in the file.
+    mode : str
+        File mode.
+
     Methods
     -------
-    close
-        Close the memmap file.
     flush
         Flush any changes in memory to file on disk.
         When you delete a memmap object, flush is called first to write
         changes to disk before removing the object.
 
+
+    See also
+    --------
+    lib.format.open_memmap : Create or load a memory-mapped ``.npy`` file.
+
     Notes
     -----
     The memmap object can be used anywhere an ndarray is accepted.
     Given a memmap ``fp``, ``isinstance(fp, numpy.ndarray)`` returns
     ``True``.
+    
+    Memory-mapped files cannot be larger than 2GB on 32-bit systems.
 
-    Memory-mapped arrays use the Python memory-map object which
-    (prior to Python 2.5) does not allow files to be larger than a
-    certain size depending on the platform. This size is always < 2GB
-    even on 64-bit systems.
+    When a memmap causes a file to be created or extended beyond its
+    current size in the filesystem, the contents of the new part are
+    unspecified. On systems with POSIX filesystem semantics, the extended
+    part will be filled with zero bytes.
 
     Examples
     --------
@@ -107,6 +142,9 @@ class memmap(ndarray):
             [  4.,   5.,   6.,   7.],
             [  8.,   9.,  10.,  11.]], dtype=float32)
 
+    >>> fp.filename == path.abspath(filename)
+    True
+
     Deletion flushes memory changes to disk before removing the object:
 
     >>> del fp
@@ -160,24 +198,31 @@ class memmap(ndarray):
     """
 
     __array_priority__ = -100.0
+
     def __new__(subtype, filename, dtype=uint8, mode='r+', offset=0,
                 shape=None, order='C'):
         # Import here to minimize 'import numpy' overhead
         import mmap
+        import os.path
         try:
             mode = mode_equivalents[mode]
         except KeyError:
             if mode not in valid_filemodes:
-                raise ValueError("mode must be one of %s" % \
-                                 (valid_filemodes + mode_equivalents.keys()))
+                raise ValueError("mode must be one of %s" %
+                                 (valid_filemodes + list(mode_equivalents.keys())))
 
-        if hasattr(filename,'read'):
+        if hasattr(filename, 'read'):
             fid = filename
+            own_file = False
+        elif is_pathlib_path(filename):
+            fid = filename.open((mode == 'c' and 'r' or mode)+'b')
+            own_file = True
         else:
-            fid = file(filename, (mode == 'c' and 'r' or mode)+'b')
+            fid = open(filename, (mode == 'c' and 'r' or mode)+'b')
+            own_file = True
 
         if (mode == 'w+') and shape is None:
-            raise ValueError, "shape must be given"
+            raise ValueError("shape must be given")
 
         fid.seek(0, 2)
         flen = fid.tell()
@@ -188,8 +233,8 @@ class memmap(ndarray):
             bytes = flen - offset
             if (bytes % _dbytes):
                 fid.close()
-                raise ValueError, "Size of available data is not a "\
-                      "multiple of data-type size."
+                raise ValueError("Size of available data is not a "
+                        "multiple of the data-type size.")
             size = bytes // _dbytes
             shape = (size,)
         else:
@@ -203,7 +248,7 @@ class memmap(ndarray):
 
         if mode == 'w+' or (mode == 'r+' and flen < bytes):
             fid.seek(bytes - 1, 0)
-            fid.write(chr(0))
+            fid.write(np.compat.asbytes('\0'))
             fid.flush()
 
         if mode == 'c':
@@ -213,25 +258,45 @@ class memmap(ndarray):
         else:
             acc = mmap.ACCESS_WRITE
 
-        if sys.version_info[:2] >= (2,6):
-            # The offset keyword in mmap.mmap needs Python >= 2.6
-            start = offset - offset % mmap.ALLOCATIONGRANULARITY
-            bytes -= start
-            offset -= start
-            mm = mmap.mmap(fid.fileno(), bytes, access=acc, offset=start)
-        else:
-            mm = mmap.mmap(fid.fileno(), bytes, access=acc)
+        start = offset - offset % mmap.ALLOCATIONGRANULARITY
+        bytes -= start
+        offset -= start
+        mm = mmap.mmap(fid.fileno(), bytes, access=acc, offset=start)
 
         self = ndarray.__new__(subtype, shape, dtype=descr, buffer=mm,
             offset=offset, order=order)
         self._mmap = mm
+        self.offset = offset
+        self.mode = mode
+
+        if isinstance(filename, basestring):
+            self.filename = os.path.abspath(filename)
+        elif is_pathlib_path(filename):
+            self.filename = filename.resolve()
+        # py3 returns int for TemporaryFile().name
+        elif (hasattr(filename, "name") and
+              isinstance(filename.name, basestring)):
+            self.filename = os.path.abspath(filename.name)
+        # same as memmap copies (e.g. memmap + 1)
+        else:
+            self.filename = None
+
+        if own_file:
+            fid.close()
+
         return self
 
     def __array_finalize__(self, obj):
-        if hasattr(obj, '_mmap'):
+        if hasattr(obj, '_mmap') and np.may_share_memory(self, obj):
             self._mmap = obj._mmap
+            self.filename = obj.filename
+            self.offset = obj.offset
+            self.mode = obj.mode
         else:
             self._mmap = None
+            self.filename = None
+            self.offset = None
+            self.mode = None
 
     def flush(self):
         """
@@ -248,37 +313,26 @@ class memmap(ndarray):
         memmap
 
         """
-        if self._mmap is not None:
-            self._mmap.flush()
+        if self.base is not None and hasattr(self.base, 'flush'):
+            self.base.flush()
 
-    def sync(self):
-        """This method is deprecated, use `flush`."""
-        warnings.warn("Use ``flush``.", DeprecationWarning)
-        self.flush()
+    def __array_wrap__(self, arr, context=None):
+        arr = super(memmap, self).__array_wrap__(arr, context)
 
-    def _close(self):
-        """Close the memmap file.  Only do this when deleting the object."""
-        if self.base is self._mmap:
-            # The python mmap probably causes flush on close, but
-            # we put this here for safety
-            self._mmap.flush()
-            self._mmap.close()
-            self._mmap = None
+        # Return a memmap if a memmap was given as the output of the
+        # ufunc. Leave the arr class unchanged if self is not a memmap
+        # to keep original memmap subclasses behavior
+        if self is arr or type(self) is not memmap:
+            return arr
+        # Return scalar instead of 0d memmap, e.g. for np.sum with
+        # axis=None
+        if arr.shape == ():
+            return arr[()]
+        # Return ndarray otherwise
+        return arr.view(np.ndarray)
 
-    def close(self):
-        """Close the memmap file. Does nothing."""
-        warnings.warn("``close`` is deprecated on memmap arrays.  Use del",
-                      DeprecationWarning)
-
-    def __del__(self):
-        # We first check if we are the owner of the mmap, rather than
-        # a view, so deleting a view does not call _close
-        # on the parent mmap
-        if self._mmap is self.base:
-            try:
-                # First run tell() to see whether file is open
-                self._mmap.tell()
-            except ValueError:
-                pass
-            else:
-                self._close()
+    def __getitem__(self, index):
+        res = super(memmap, self).__getitem__(index)
+        if type(res) is memmap and res._mmap is None:
+            return res.view(type=ndarray)
+        return res
