@@ -4,14 +4,12 @@
 
 #define NPY_NO_DEPRECATED_API NPY_API_VERSION
 #define _MULTIARRAYMODULE
-
 #include "numpy/arrayobject.h"
 #include "numpy/arrayscalars.h"
 #include "numpy/arrayobject.h"
 
 #include "npy_config.h"
 #include "npy_pycompat.h"
-#include "npy_import.h"
 
 #include "common.h"
 #include "arraytypes.h"
@@ -537,6 +535,13 @@ PyArray_OrderConverter(PyObject *object, NPY_ORDER *val)
         PyObject *tmp;
         int ret;
         tmp = PyUnicode_AsASCIIString(object);
+        if (tmp == NULL) {
+            PyErr_SetString(PyExc_ValueError, "Invalid unicode string passed in "
+                                              "for the array ordering. "
+                                              "Please pass in 'C', 'F', 'A' "
+                                              "or 'K' instead");
+            return NPY_FAIL;
+        }
         ret = PyArray_OrderConverter(tmp, val);
         Py_DECREF(tmp);
         return ret;
@@ -799,28 +804,17 @@ PyArray_PyIntAsIntp_ErrMsg(PyObject *o, const char * msg)
     long long_value = -1;
 #endif
     PyObject *obj, *err;
-    static PyObject *VisibleDeprecation = NULL;
 
-    npy_cache_import(
-        "numpy", "VisibleDeprecationWarning", &VisibleDeprecation);
-
-    if (!o) {
+    /*
+     * Be a bit stricter and not allow bools.
+     * np.bool_ is also disallowed as Boolean arrays do not currently
+     * support index.
+     */
+    if (!o || PyBool_Check(o) || PyArray_IsScalar(o, Bool)) {
         PyErr_SetString(PyExc_TypeError, msg);
         return -1;
     }
 
-    /* Be a bit stricter and not allow bools, np.bool_ is handled later */
-    if (PyBool_Check(o)) {
-        /* 2013-04-13, 1.8 */
-        if (PyErr_WarnEx(
-                VisibleDeprecation,
-                "using a boolean instead of an integer"
-                " will result in an error in the future",
-                1) < 0) {
-            return -1;
-        }
-    }
-
     /*
      * Since it is the usual case, first check if o is an integer. This is
      * an exact check, since otherwise __index__ is used.
@@ -846,90 +840,22 @@ PyArray_PyIntAsIntp_ErrMsg(PyObject *o, const char * msg)
         return (npy_intp)long_value;
     }
 
-    /* Disallow numpy.bool_. Boolean arrays do not currently support index. */
-    if (PyArray_IsScalar(o, Bool)) {
-        /* 2013-06-09, 1.8 */
-        if (PyErr_WarnEx(
-                VisibleDeprecation,
-                "using a boolean instead of an integer"
-                " will result in an error in the future",
-                1) < 0) {
-            return -1;
-        }
-    }
-
     /*
      * The most general case. PyNumber_Index(o) covers everything
      * including arrays. In principle it may be possible to replace
      * the whole function by PyIndex_AsSSize_t after deprecation.
      */
     obj = PyNumber_Index(o);
-    if (obj) {
-#if (NPY_SIZEOF_LONG < NPY_SIZEOF_INTP)
-        long_value = PyLong_AsLongLong(obj);
-#else
-        long_value = PyLong_AsLong(obj);
-#endif
-        Py_DECREF(obj);
-        goto finish;
-    }
-    else {
-        /*
-         * Set the TypeError like PyNumber_Index(o) would after trying
-         * the general case.
-         */
-        PyErr_Clear();
-    }
-
-    /*
-     * For backward compatibility check the number C-Api number protcol
-     * This should be removed up the finish label after deprecation.
-     */
-    if (Py_TYPE(o)->tp_as_number != NULL &&
-        Py_TYPE(o)->tp_as_number->nb_int != NULL) {
-        obj = Py_TYPE(o)->tp_as_number->nb_int(o);
-        if (obj == NULL) {
-            return -1;
-        }
- #if (NPY_SIZEOF_LONG < NPY_SIZEOF_INTP)
-        long_value = PyLong_AsLongLong(obj);
- #else
-        long_value = PyLong_AsLong(obj);
- #endif
-        Py_DECREF(obj);
-    }
-#if !defined(NPY_PY3K)
-    else if (Py_TYPE(o)->tp_as_number != NULL &&
-             Py_TYPE(o)->tp_as_number->nb_long != NULL) {
-        obj = Py_TYPE(o)->tp_as_number->nb_long(o);
-        if (obj == NULL) {
-            return -1;
-        }
-  #if (NPY_SIZEOF_LONG < NPY_SIZEOF_INTP)
-        long_value = PyLong_AsLongLong(obj);
-  #else
-        long_value = PyLong_AsLong(obj);
-  #endif
-        Py_DECREF(obj);
-    }
-#endif
-    else {
-        PyErr_SetString(PyExc_TypeError, msg);
+    if (obj == NULL) {
         return -1;
     }
-    /* Give a deprecation warning, unless there was already an error */
-    if (!error_converting(long_value)) {
-        /* 2013-04-13, 1.8 */
-        if (PyErr_WarnEx(
-                VisibleDeprecation,
-                "using a non-integer number instead of an integer"
-                " will result in an error in the future",
-                1) < 0) {
-            return -1;
-        }
-    }
+#if (NPY_SIZEOF_LONG < NPY_SIZEOF_INTP)
+    long_value = PyLong_AsLongLong(obj);
+#else
+    long_value = PyLong_AsLong(obj);
+#endif
+    Py_DECREF(obj);
 
- finish:
     if (error_converting(long_value)) {
         err = PyErr_Occurred();
         /* Only replace TypeError's here, which are the normal errors. */
@@ -938,9 +864,9 @@ PyArray_PyIntAsIntp_ErrMsg(PyObject *o, const char * msg)
         }
         return -1;
     }
-
     goto overflow_check; /* silence unused warning */
- overflow_check:
+
+overflow_check:
 #if (NPY_SIZEOF_LONG < NPY_SIZEOF_INTP)
   #if (NPY_SIZEOF_LONGLONG > NPY_SIZEOF_INTP)
     if ((long_value < NPY_MIN_INTP) || (long_value > NPY_MAX_INTP)) {
